# Generated by Django 4.2.15 on 2025-05-05 11:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0015_alter_corpinvoice_tax_reason'),
        ('corptax', '0015_discordnotification'),
    ]

    operations = [
    ]



# Generated by Django 4.2.16 on 2025-05-03 11:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0014_troikavie_alter_adminview_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='DiscordNotification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_sent', models.BooleanField(default=False, help_text='True when this notification has been sent to Discord', verbose_name='is sent')),
                ('owner', models.IntegerField(help_text='Owner of the message', verbose_name='owner')),
                ('time_sent', models.DateTimeField(default=True, help_text='The time when the message has been sent')),
                ('discord_msg', models.CharField(help_text='The message what been sent to Discord', max_length=128, verbose_name='discord_msg')),
            ],
        ),
    ]



import logging
from inspect import getmembers, isfunction
from datetime import datetime

from django.core.management.base import BaseCommand
from django.conf import settings

from allianceauth.authentication.models import CharacterOwnership, OwnershipRecord, UserProfile
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from esi.clients import EsiClientProvider
from app_utils.esi import fetch_esi_status
from corptools.models import CorporationWalletJournalEntry
from memberaudit.models import Character as AuditCharacter
from corptax.models import CorpStats

logger = logging.getLogger(__name__)
#[15:46:22] Dejar Winter > <url=showinfo:14//40257464>XZH-4X VII - Moon 9</url>
#[16:49:37] Dejar Winter > <url=showinfo:14//40257071>FQ9W-C VII - Moon 13</url>
#### Moon Taxes
class Command(BaseCommand):
    help = 'Generate Moon Rental'
    def handle(self, *args, **options):
        if not fetch_esi_status().is_ok:
            print(f'ESI not working')
            quit(1)
        current_date = datetime.now()
        year = current_date.strftime("%Y")
        month = current_date.strftime("%m")
        start_date = str(year) + "-" + str(month) + "-01 00:00"
        end_date = str(year) + "-" + str(month) + "-30 00:00"
        accounted_alliance = [99007906, 741557221]
        esi = EsiClientProvider()
        alliance_corps = []
        for alliance in accounted_alliance:
            alliance_corps = []
            corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
            for x in corps:
                alliance_corps.append(x)
            for corp_id in alliance_corps:
                auth_corp_info = EveCorporationInfo.objects.get(corporation_id=corp_id)
                auth_alliance_info = EveAllianceInfo.objects.get(id=auth_corp_info.alliance_id)
                print(vars(auth_alliance_info))
                print(vars(auth_corp_info))
                esi_corp_info = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
                jornal = CorporationWalletJournalEntry.objects.filter(
                    division__corporation__corporation__corporation_id=auth_corp_info.corporation_id, 
                    date__gte=start_date, date__lte=end_date
                )
                corp_tax = round(esi_corp_info['tax_rate'], 2) * 100
                check_corp_journal = 0
                check_corp_ceo = 0
                check_corp_tax = 0
                check_audit_member = 0
                if "RZR Vote Corp" not in auth_corp_info.corporation_name:
                    if len(jornal) < 1:
                        check_corp_journal = 1
                    try:
                        corp_ceo = EveCharacter.objects.get(character_id=auth_corp_info.ceo_id)
                    except Exception as e:
                        check_corp_ceo = 1
                    if corp_tax >= 50 or corp_tax < 5:
                        check_corp_tax = 1
                    if check_corp_ceo == 1:
                        logger.info(f'Ceo character not in Auth')
                    if check_corp_journal == 1:
                        logger.info(f'has no Corp Wallet entry\'s')
                    if check_corp_tax == 1:
                        logger.info(f'Tax settings wrong: {corp_tax}%')
                    auth_members = EveCharacter.objects.filter(corporation_id=corp_id)
                    for member in auth_members:
                        try:
                            audit_character = AuditCharacter.objects.get(eve_character_id=member.id)
                            check_audit_member = check_audit_member + 1
                        except:
                            logger.info(f'{member} not in Audit')

                    try:
                        entry = CorpStats.objects.get(corp_id=corp_id)
                        entry.auth_member = len(auth_members)
                        entry.audit_member = check_audit_member
                        entry.corp_tax = corp_tax
                        entry.auth_ceo = check_corp_ceo
                        entry.corp_journal = check_corp_journal
                        entry.total_member = auth_corp_info.member_count
                        #entry.save()
                    except:
                        #CorpStats.objects.create(corp_id=corp_id, corp_tax=corp_tax, auth_member=len(auth_members), 
                        #    audit_member=check_audit_member, auth_ceo=check_corp_ceo, corp_journal=check_corp_journal,
                        #    corp_name=auth_corp_info.corporation_name, total_member=auth_corp_info.member_count
                        #)
                        print("Nothing")



import logging
from django.core.management.base import BaseCommand
from structures.models import Structure, StructureTag
logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'Fuel calculator'
    def handle(self, *args, **options):
        print("Hello")
        structures = Structure.objects.all()
        for structure in structures:
            print(vars(structure))


import logging
import csv
import operator as op
from inspect import getmembers, isfunction
from datetime import datetime, timedelta, date
from dateutil.relativedelta import relativedelta

from django.core.management.base import BaseCommand
from django.utils import timezone
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag
from esi.clients import EsiClientProvider
from eveuniverse.models import EveType, EveTypeMaterial, EveMarketPrice
from moonmining.models import Moon, MoonProduct

from moonmining.models import MiningLedgerRecord as Ledger

from corptax.helpers import unique, lookup_invoice, generate_invoice, get_ore_rarity_tax, get_ratting_tax, generate_tax_preview

from corptax.models import TaxPreview

logger = logging.getLogger(__name__)
#[15:46:22] Dejar Winter > <url=showinfo:14//40257464>XZH-4X VII - Moon 9</url>
#[16:49:37] Dejar Winter > <url=showinfo:14//40257071>FQ9W-C VII - Moon 13</url>
#### Moon Taxes
class Command(BaseCommand):
    help = 'Generate Moon Rental'
    def handle(self, *args, **options):
        accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)
        current_date = datetime.now()
        year = current_date.strftime("%Y")
        month = current_date.strftime("%m")
        esi = EsiClientProvider()
        tax = 0.05
        logger.info(f'Starting Moon Rental calculation')
        #moons = Moon.objects.filter(eve_moon="6Z9-0M VI - 9")
        #moons = Moon.objects.filter(eve_moon_id=40257071)
        moons = Moon.objects.all()
        for moon in moons:
            print("")
            print(f'Moon: {str(moon)}')
            moon_product = MoonProduct.objects.filter(moon_id=moon.eve_moon_id)
            total_moon_value = 0
            for x in moon_product:
                #print(vars(x))
                #ore_output_volume = 29196160 * x.amount
                ore_output_volume = 21888000 * x.amount
                ore_output = ore_output_volume / 10
                ore_material = EveTypeMaterial.objects.filter(eve_type=x.ore_type_id)
                for y in ore_material:
                    ore_material_value = 0
                    material_type = EveType.objects.get(id=y.material_eve_type_id)
                    if material_type.eve_group_id not in [427]:
                        continue
                    total_minerales = ore_output * y.quantity / 100 * 0.8
                    input_mats_name = str(EveType.objects.get(id=y.material_eve_type_id))
                    price = EveMarketPrice.objects.get(eve_type=y.material_eve_type_id)
                    ore_material_value = total_minerales * price.average_price
                    total_moon_value = total_moon_value + ore_material_value
                    #print(input_mats_name, round(total_minerales), ore_material_value)
                    print(input_mats_name, round(total_minerales))
            print(f'Total moon value: {round(total_moon_value)}')





# Generated by Django 4.2.15 on 2024-09-27 10:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0002_corpinvoice'),
    ]

    operations = [
        migrations.CreateModel(
            name='AdminView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('admin_access', 'Can view Corp Taxes'),),
                'managed': False,
                'default_permissions': (),
            },
        ),
    ]



# Generated by Django 4.2.15 on 2024-12-06 12:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0011_alter_alliancefinance_expense_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='alliancefinance',
            name='planatary_income',
        ),
    ]



import logging

from django.core.management.base import BaseCommand
from django.conf import settings
from esi.models import Token
from allianceauth.framework.api.evecharacter import get_user_from_evecharacter

logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'Generate Moon Drill Rental'
    def handle(self, *args, **options):
        braten = Token.objects.filter(character_id=96663489).require_valid()
        print(vars(braten))
        for i in braten:
            print(i)
        



# Generated by Django 4.2.15 on 2025-05-30 07:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0016_merge_20250505_1148'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Invoice',
        ),
        migrations.DeleteModel(
            name='TaxPreview',
        ),
    ]



"""App URLs"""

# Django
from django.urls import path

# AA Example App
from corptax import views

app_name: str = "corptax"

urlpatterns = [
    path("",
         views.index,
         name="index"
         ),
    path("moon_member/<int:corp_id>/<int:tax_month>/<int:tax_year>",
         views.moon_member,
         name="moon_member"
         ),
    path("tax/<int:tax_month>/<int:tax_year>",
         views.month_tax_view,
         name="corp tax view"
         ),
    path("data-export/<int:corp_id>/member-ledger.csv",
         views.download_corp_member_ledger,
         name="Download Corp member ledger"
         ),
     path("corpstats",
         views.view_corp_stats,
         name="Corp Stats"
         ),
     path("finance/<int:month>/<int:year>",
         views.view_alliance_finance,
         name="Alliance Finance"
         ),
     path("finance",
         views.view_alliance_finance_current,
         name="Alliance Finance"
         ),
]



"""App Tasks"""

from celery import shared_task

from datetime import datetime, timedelta
from django.conf import settings

from allianceauth.services.hooks import get_extension_logger
from app_utils.esi import fetch_esi_status


from .helpers import finance_calculation, young_cyno_chars
from corptax.tax import generate_tax_ratting, generate_tax_moonmining, generate_tax_moondrill, generate_tax_moonathanor, generate_corp_stats, check_corp_tax

from . import __title__
logger = get_extension_logger(__name__)

#########################################################
# Task, generate moon tax invoice
#########################################################
@shared_task
def task_moon_tax():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_tax')
        return(False)
    else:
        today = datetime.today()
        first = today.replace(day=1)
        last_month = first - timedelta(days=1)
        end_date = last_month.replace(hour=23, minute=59)
        start_date = last_month.replace(day=1, hour=00, minute=00)

        logger.info(f'Starting Moon Mining Tax calculation {start_date} {end_date}')
        bill = generate_tax_moonmining(start_date, end_date, notify=True)
        logger.info(f'Finished Moon Mining Tax calculation {start_date} {end_date}')

#########################################################
# Task generate moon tax preview
#########################################################
@shared_task
def task_moon_tax_preview():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_tax_preview')
        return(False)
    else:
        today = datetime.today()
        start_date = today.replace(day=1, hour=00, minute=00)
        end_date = today

        logger.info(f'Start preview Moon Mining Tax calculation {start_date} {end_date}')
        bill = generate_tax_moonmining(start_date, end_date, notify=False)
        logger.info(f'Finished preview Moon Mining Tax calculation {start_date} {end_date}')


#########################################################
# Task, generate ratting tax invoice
#########################################################
@shared_task
def task_ratting_tax():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_ratting_tax')
        return(False)
    else:
        today = datetime.today()
        first = today.replace(day=1)
        last_month = first - timedelta(days=1)
        end_date = last_month.replace(hour=23, minute=59)
        start_date = last_month.replace(day=1, hour=00, minute=00)

        logger.info(f'Starting Rattix Tax calculation {start_date} {end_date}')
        bill = generate_tax_ratting(start_date, end_date, notify=True)
        logger.info(f'Finished Rattix Tax calculation {start_date} {end_date}')
    
    

#########################################################
# Task, generate ratting tax preview
#########################################################
@shared_task
def task_ratting_tax_preview():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_ratting_tax_preview')
        return(False)
    else:
        today = datetime.today()
        start_date = today.replace(day=1, hour=00, minute=00)
        end_date = today

        logger.info(f'Starting Ratting Tax preview calculation {start_date} {end_date}')
        bill = generate_tax_ratting(start_date, end_date, notify=False)
        logger.info(f'Finished Ratting Tax preview calculation {start_date} {end_date}')
    

#########################################################
# Task, generate moon drill invoice
#########################################################
@shared_task
def task_moon_drill_tax():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_drill_tax')
        return(False)
    else:
        today = datetime.today()
        first = today.replace(day=1)
        last_month = first - timedelta(days=1)
        end_date = last_month.replace(hour=23, minute=59)
        start_date = last_month.replace(day=1, hour=00, minute=00)
        logger.info(f'Starting Moon Drill Tax calculation for date {start_date}/{end_date}')
        generate_tax_moondrill(start_date, end_date, notify=True)
        logger.info(f'Finished Moon Drill Tax calculation for date {start_date}/{end_date}')

#########################################################
# Task, generate moon drill preview
#########################################################
@shared_task
def task_moon_drill_tax_preview():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_drill_tax_preview')
        return(False)
    else:
        today = datetime.today()
        start_date = today.replace(day=1, hour=00, minute=00)
        end_date = today
        logger.info(f'Starting preview Moon Drill Tax calculation for date {start_date}/{end_date}')
        generate_tax_moondrill(start_date, end_date, notify=False)
        logger.info(f'Finished preview Moon Drill Tax calculation for date {start_date}/{end_date}')


#########################################################
# Task, generate moon athanor invoice
#########################################################
@shared_task
def task_moon_athanor_tax():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_athanor_tax')
        return(False)
    else:
        today = datetime.today()
        first = today.replace(day=1)
        last_month = first - timedelta(days=1)
        end_date = last_month.replace(hour=23, minute=59)
        start_date = last_month.replace(day=1, hour=00, minute=00)
        logger.info(f'Starting Moon Athanor Tax calculation for date {start_date}/{end_date}')
        generate_tax_moonathanor(start_date, end_date, notify=True)
        logger.info(f'Finished Moon Athanor Tax calculation for date {start_date}/{end_date}')

#########################################################
# Task, generate moon athanor preview
#########################################################
@shared_task
def task_moon_athanor_tax_preview():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_moon_athanor_tax_preview')
        return(False)
    else:
        today = datetime.today()
        start_date = today.replace(day=1, hour=00, minute=00)
        end_date = today
        logger.info(f'Starting preview Moon Athanor Tax calculation for date {start_date}/{end_date}')
        generate_tax_moonathanor(start_date, end_date, notify=False)
        logger.info(f'Finished preview Moon Athanor Tax calculation for date {start_date}/{end_date}')

#########################################################
# Task, generate corp stats
#########################################################

@shared_task
def task_corp_stats_update():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_corp_stats_update')
        return(False)
    else:
        today = datetime.today()
        start_date = today.replace(day=1, hour=00, minute=00)
        end_date = today
        logger.info(f'Starting generating corp stats {start_date}/{end_date}')
        generate_corp_stats(start_date, end_date, notify=False)
        logger.info(f'Finished generating corp stats {start_date}/{end_date}')


#########################################################
# Task, generate Alliance Finance current month
#########################################################
@shared_task
def task_alliance_finance_current():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_alliance_finance_current')
        return(False)
    else:
        today = datetime.today()
        current_month_start_date = today.replace(day=1, hour=00, minute=00)
        current_month_end_date = today
        logger.info(f'Starting Alliance Finance calculation for current month {current_month_start_date}/{current_month_end_date}')
        run = finance_calculation(current_month_start_date, current_month_end_date)
        logger.info(f'Finished Alliance Finance calculation for current month {current_month_start_date}/{current_month_end_date}')


#########################################################
# Task, generate Alliance Finance 
#########################################################
@shared_task
def task_alliance_finance():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_alliance_finance')
        return(False)
    else:
        today = datetime.today()
        first = today.replace(day=1)
        last_month = first - timedelta(days=1)
        last_month_end_date = last_month.replace(hour=23, minute=59)
        last_month_start_date = last_month.replace(day=1, hour=00, minute=00)
        logger.info(f'Starting Alliance Finance calculation for last month {last_month_start_date}/{last_month_end_date}')
        run = finance_calculation(last_month_start_date, last_month_end_date)
        logger.info(f'Finished Alliance Finance calculation for last month {last_month_start_date}/{last_month_end_date}')


#########################################################
# Task, check for corp tax setting and alert via discord
#########################################################
@shared_task
def task_check_corp_tax():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_check_corp_tax')
        return(False)
    else:
        logger.info(f'Starting check corp tax setting')
        check_corp_tax()
        logger.info(f'Finished check corp tax setting')

#########################################################
# Task, check for young cyno V chars
#########################################################
@shared_task
def task_check_cyno_chars():
    if not fetch_esi_status().is_ok:
        logger.warning(f'ESI not working, exiting task_check_cyno_chars')
        return(False)
    else:
        logger.info(f'Starting check for young cyno chars')
        young_cyno_chars()
        logger.info(f'Finished check for young cyno chars')


# Generated by Django 4.2.15 on 2024-12-06 12:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0010_alliancefinance'),
    ]

    operations = [
        migrations.AlterField(
            model_name='alliancefinance',
            name='expense',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True, verbose_name='expense'),
        ),
        migrations.AlterField(
            model_name='alliancefinance',
            name='income',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=20, null=True, verbose_name='income'),
        ),
    ]



# Generated by Django 4.2.15 on 2025-05-02 07:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0014_troikavie_alter_adminview_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='corpinvoice',
            name='tax_reason',
            field=models.CharField(max_length=128, verbose_name='tax_reason'),
        ),
    ]



# Generated by Django 4.2.15 on 2024-09-28 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0003_adminview'),
    ]

    operations = [
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corp_id', models.BigIntegerField(verbose_name='corp_id')),
                ('date', models.DateField(verbose_name='date')),
            ],
        ),
        migrations.CreateModel(
            name='MoonLedgerMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corp_id', models.BigIntegerField(verbose_name='corp_id')),
                ('character_id', models.BigIntegerField(verbose_name='character_id')),
                ('date', models.DateField(verbose_name='date')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Amount')),
            ],
        ),
    ]



"""App Views"""

from allianceauth.services.hooks import get_extension_logger
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse, FileResponse, Http404
from django.shortcuts import render
from django.utils.translation import gettext_lazy as _
from allianceauth.services.hooks import get_extension_logger
from corptax.models import CorpInvoice, MoonLedgerMember, CorpStats, AllianceFinance
from corptax.helpers import download_corp_member_ledger_csv
from datetime import date
import os


from . import __title__
logger = get_extension_logger(__name__)

@login_required
@permission_required("corptax.basic_access")
def index(request: WSGIRequest) -> HttpResponse:
    """Render the main tax page with current month's tax records."""
    current_date = date.today()
    year = current_date.year
    month = current_date.month
    tax_record_moon = CorpInvoice.objects.filter(tax_reason__contains='moon', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
    tax_record_ratting = CorpInvoice.objects.filter(tax_reason__contains='ratting', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
    tax_record_drill = CorpInvoice.objects.filter(tax_reason__contains='DRILL', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
    tax_record_athanor = CorpInvoice.objects.filter(tax_reason__contains='ATHANOR_', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
    tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
    tax_date = str(month) + "/" + str(year)
    total_moon = 0
    for entry in tax_record_moon:
        total_moon = total_moon + entry.amount
    total_ratting = 0
    for entry in tax_record_ratting:
        total_ratting = total_ratting + entry.amount
    total_drill = 0
    for entry in tax_record_drill:
        total_drill = total_drill + entry.amount
    total_athanor = 0
    for entry in tax_record_athanor:
        total_athanor = total_athanor + entry.amount
     
    context = {
        'tax_preview_moon' :tax_record_moon,
        'tax_preview_ratting' :tax_record_ratting,
        'tax_preview_drill' :tax_record_drill,
        'tax_preview_athanor' :tax_record_athanor,
        'tax_moon_total': total_moon,
        'tax_ratting_total': total_ratting,
        'tax_athanor_total': total_athanor,
        'tax_drill_total': total_drill,
        'tax_date': tax_date,
        'tax_month_list': tax_month_list,
    }
    return render(request, "corptax/index.html", context)

@login_required
@permission_required("corptax.basic_access")
def month_tax_view(request, tax_month, tax_year):
    """Render the tax view for a specific month and year."""
    tax_corp_moon = CorpInvoice.objects.filter(tax_reason__contains='moon', tax_date__month=tax_month, tax_date__year=tax_year).order_by('-tax_date', 'corp_name')
    tax_corp_ratting = CorpInvoice.objects.filter(tax_reason__contains='ratting', tax_date__month=tax_month, tax_date__year=tax_year).order_by('-tax_date', 'corp_name')
    tax_corp_drill = CorpInvoice.objects.filter(tax_reason__contains='DRILL', tax_date__month=tax_month, tax_date__year=tax_year).order_by('-tax_date', 'corp_name')
    tax_corp_athanor = CorpInvoice.objects.filter(tax_reason__contains='ATHANOR_', tax_date__month=tax_month, tax_date__year=tax_year).order_by('-tax_date', 'corp_name')
    tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
    tax_date = str(tax_month) + "/" + str(tax_year)
    total_moon = 0
    for entry in tax_corp_moon:
        total_moon = total_moon + entry.amount
    total_ratting = 0
    for entry in tax_corp_ratting:
        total_ratting = total_ratting + entry.amount
    total_drill = 0
    for entry in tax_corp_drill:
        total_drill = total_drill + entry.amount
    total_athanor = 0
    for entry in tax_corp_athanor:
        total_athanor = total_athanor + entry.amount

    total_tax = total_moon + total_ratting + total_drill
    
    context = {
        'tax_corp_moon': tax_corp_moon,
        'tax_corp_ratting': tax_corp_ratting,
        'tax_date': tax_date,
        'tax_corp_drill': tax_corp_drill,
        'tax_moon_total': total_moon,
        'tax_ratting_total': total_ratting,
        'tax_drill_total': total_drill,
        'total_tax': total_tax,
        'tax_month_list': tax_month_list,
        'tax_corp_athanor': tax_corp_athanor,
        'tax_athanor_total': total_athanor,
    }
    return render(request, "corptax/corptax.html", context)

@login_required
@permission_required("corptax.basic_access")
def moon_member(request, corp_id, tax_month, tax_year):
    moon_ledger_member = MoonLedgerMember.objects.filter(corp_id=corp_id, date__month=tax_month, date__year=tax_year)
    
    context = {
        'corp_id': corp_id,
        'moon_ledger_member': moon_ledger_member
    }
    return render(request, "corptax/membermoon.html", context)

@login_required
@permission_required("corptax.basic_access")
def download_corp_member_ledger(request, corp_id):
    try:
        download_file = download_corp_member_ledger_csv(corp_id)
        download = "/tmp/" + str(corp_id) + ".csv"
        os.rename(download_file.name, download)
    except:
        logger.error(f'Could not find export file for {corp_id}')
        raise Http404(f"Could not find export file for {corp_id}")
    logger.info(f'Returning corp member ledger csv for corp {corp_id}')
    return FileResponse(open(download, "rb"))

@login_required
@permission_required("corptax.basic_access")
def view_corp_stats(request):
    tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
    corp_stats_edge = CorpStats.objects.filter(alliance_id=99007906)
    corp_stats_razor = CorpStats.objects.filter(alliance_id=741557221)
    context = {
        'corp_stats_edge': corp_stats_edge,
        'corp_stats_razor': corp_stats_razor,
        'tax_month_list': tax_month_list,
    }
    return render(request, "corptax/corpstats.html", context)

@login_required
@permission_required("corptax.troika_access")
def view_alliance_finance(request,month, year):
    display_date = str(month) + "/" + str(year)
    tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
    finance_income = AllianceFinance.objects.filter(date__year=year, date__month=month, income__gt=0)
    finance_expense = AllianceFinance.objects.filter(date__year=year, date__month=month, expense__lt=0)
    finance_months = AllianceFinance.objects.order_by().values_list('date', flat=True).distinct().order_by('date')
    income_total = 0
    expense_total = 0
    for x in finance_income:
        income_total = income_total + x.income
    for x in finance_expense:
        expense_total = expense_total + x.expense
    total = income_total + expense_total

    context = {
        'tax_month_list': tax_month_list,
        'finance_income': finance_income,
        'finance_expense': finance_expense,
        'income_total': income_total,
        'expense_total': expense_total,
        'total': total,
        'date': display_date,
        'months': finance_months
    }
    return render(request, "corptax/finance.html", context)

@login_required
@permission_required("corptax.troika_access")
def view_alliance_finance_current(request):
    current_date = date.today()
    year = current_date.year
    month = current_date.month
    display_date = str(month) + "/" + str(year)
    tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
    finance_income = AllianceFinance.objects.filter(date__year=year, date__month=month, income__gt=0)
    finance_expense = AllianceFinance.objects.filter(date__year=year, date__month=month, expense__lt=0)
    finance_months = AllianceFinance.objects.order_by().values_list('date', flat=True).distinct()
    income_total = 0
    expense_total = 0
    for x in finance_income:
        income_total = income_total + x.income
    for x in finance_expense:
        expense_total = expense_total + x.expense
    total = income_total + expense_total

    context = {
        'tax_month_list': tax_month_list,
        'finance_income': finance_income,
        'finance_expense': finance_expense,
        'income_total': income_total,
        'expense_total': expense_total,
        'total': total,
        'date': display_date,
        'months': finance_months
    }
    return render(request, "corptax/finance.html", context)



# Generated by Django 4.2.15 on 2024-11-23 17:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0008_corpstats_corp_name_corpstats_total_member'),
    ]

    operations = [
        migrations.AddField(
            model_name='corpstats',
            name='alliance_id',
            field=models.BigIntegerField(null=True, verbose_name='corp_id'),
        ),
    ]



import logging
import datetime
from django.core.management.base import BaseCommand
from corptax.helpers import finance_calculation
from corptax.models import AllianceFinance
from corptools.models import CorporationWalletJournalEntry, CorporationWalletDivision, CorporationAudit
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo
logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'Income and expense'
    def handle(self, *args, **options):
        today = datetime.datetime.today()
        first = today.replace(day=1)
        current_month_start_date = today.replace(day=1, hour=00, minute=00)
        current_month_end_date = today
        last_month = first - datetime.timedelta(days=1)
        last_month_end_date = last_month.replace(hour=23, minute=59)
        last_month_start_date = last_month.replace(day=1, hour=00, minute=00)
        #run = finance_calculation(last_month_start_date, last_month_end_date)
        #print(f'current {current_month_start_date} {current_month_end_date}')
        #print(f'last {last_month_start_date} {last_month_end_date}')
        cd = AllianceFinance.objects.order_by().values_list('date', flat=True).distinct()
        corp_id = 158202185
        # Get Alliance Auth corp info
        aa_corps = EveCorporationInfo.objects.filter(corporation_id=corp_id)
        aa_corp = aa_corps[0]
        #Get corptools copr info
        corptools_corps = CorporationAudit.objects.filter(corporation_id=aa_corp.id)
        corptools_corp = corptools_corps[0]
        #Get Division for selected Corp
        divisions = CorporationWalletDivision.objects.filter(corporation_id=corptools_corp.id)
        for division in divisions:
            print(vars(division))

        #### Generate for the past
        #for cd in "23456":
        #    print(last_month)
        #    first = last_month.replace(day=1)
        #    last_month = first - datetime.timedelta(days=1)
        #    last_month_end_date = last_month.replace(hour=23, minute=59)
        #    last_month_start_date = last_month.replace(day=1, hour=00, minute=00)
        #    print(f'{last_month_start_date} / {last_month_end_date}')
        #    finance_calculation(last_month_start_date, last_month_end_date)




"""App Tasks"""


import logging
import csv
import operator as op
from inspect import getmembers, isfunction
from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta

from django.core.management.base import BaseCommand
from django.utils import timezone
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag
from esi.clients import EsiClientProvider
from eveuniverse.models import EveType, EveGroup

from corptools.models import CorporationWalletJournalEntry

from moonmining.models.moons import Moon as MoonMiningMoon
from moonmining.models.owners import Refinery
from moonmining.models import MiningLedgerRecord as Ledger, OreRarityClass

logger = logging.getLogger(__name__)

#### Moon Taxes
class Command(BaseCommand):
    help = 'Generate Moon Tax break down'
    def handle(self, *args, **options):
        def unique(list1):
            unique_list = []
            for x in list1:
                if op.countOf(unique_list, x) == 0:
                    unique_list.append(x)
            return(unique_list)

        def get_ore_rarity_tax(type_id):
            ore_type = EveType.objects.get(id=type_id)
            if ore_type.eve_group_id == 1923:
                tax = 0.35
            elif ore_type.eve_group_id == 1922:
                tax = 0.15
            elif ore_type.eve_group_id == 1921:
                tax = 0.025
            elif ore_type.eve_group_id == 1920:
                tax = 0.025
            elif ore_type.eve_group_id == 1884:
                tax = 0.025
            else:
                tax = 0
            return(tax)

        last_month = datetime.now() - relativedelta(months=1)
        last_year = last_month.strftime("%Y")
        last_month = last_month.strftime("%m")
        esi = EsiClientProvider()
        csv_filename = "/tmp/report_moontax_" + str(last_year) + str(last_month) + ".csv"

        try:
            alliance_corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=741557221).results()
        except:
            logger.warning(f'Failed to make Alliance ESI query, existing')
            exit(1)

        ledger_month = Ledger.objects.filter(day__year=last_year, day__month=last_month)
        refinery_list = []
        report = []
        for ledger in ledger_month:
            refinery_list.append(ledger.refinery_id)
       
        refinerys = unique(refinery_list)
        for refinery in refinerys:
            for corp_id in alliance_corps:
                ledger_refinery = Ledger.objects.filter(day__year=last_year, day__month=last_month, refinery_id=refinery, corporation_id=corp_id)
                for entry in ledger_refinery:
                    ore_type = EveType.objects.get(id=entry.ore_type_id)
                    ore_name = str(ore_type)
                    tax_value = get_ore_rarity_tax(entry.ore_type_id)
                    character = esi.client.Character.get_characters_character_id(character_id=entry.character_id).results()
                    corporation = esi.client.Corporation.get_corporations_corporation_id(corporation_id=entry.corporation_id).results()
                    refinery_name = Refinery.objects.get(id=entry.refinery_id)
                    tax_amount = entry.total_price * tax_value
                    report_dict = {"refinery": refinery_name.name, "date": str(entry.day), "corporation": str(corporation['name']), "character": str(character['name']), "ore_type": ore_name, 
                                          "quantity": entry.quantity, "total_price": round(entry.total_price, 2), "tax_amount": round(tax_amount, 2)}
                    report.append(report_dict)
                    
        fields = ['refinery', 'date', 'corporation', 'character', 'ore_type', 'quantity', 'total_price', 'tax_amount']
        with open(csv_filename, 'w') as file:
            writer = csv.DictWriter(file, fieldnames=fields, dialect='excel')
            writer.writeheader()
            writer.writerows(report)
            



# Generated by Django 4.2.15 on 2024-09-23 17:14

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('basic_access', 'Can access this app'),),
                'managed': False,
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='TaxPreview',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Amount')),
                ('corp_name', models.CharField(max_length=128, verbose_name='CorpName')),
                ('tax_reason', models.CharField(max_length=128, verbose_name='CorpName')),
                ('corp_id', models.BigIntegerField(verbose_name='corp_id')),
                ('tax_date', models.DateField(verbose_name='tax_date')),
            ],
            options={
                'verbose_name': 'Corp Tax',
                'verbose_name_plural': 'Corp Taxes',
                'ordering': ['corp_name'],
            },
        ),
    ]



# Generated by Django 4.2.15 on 2024-12-07 11:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0012_remove_alliancefinance_planatary_income'),
    ]

    operations = [
        migrations.AddField(
            model_name='alliancefinance',
            name='description',
            field=models.CharField(blank=True, max_length=128, null=True, verbose_name='reason'),
        ),
    ]



# Generated by Django 4.2.15 on 2024-12-06 11:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0009_corpstats_alliance_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllianceFinance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(verbose_name='date')),
                ('planatary_income', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='planatary_income')),
                ('income', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='income')),
                ('expense', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='expense')),
                ('reason', models.CharField(max_length=128, verbose_name='reason')),
            ],
        ),
    ]



from datetime import datetime, timedelta
from dateutil.relativedelta import relativedelta
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo, EveAllianceInfo
from allianceauth.services.hooks import get_extension_logger
from esi.clients import EsiClientProvider

from moonmining.models import MiningLedgerRecord as Ledger
from moonmining.models import MoonProduct
from memberaudit.models import Character as AuditCharacter
from corptools.models import CorporationWalletJournalEntry
from structures.models import Structure, StructureTag
from eveuniverse.models import EveType, EveTypeMaterial, EveMarketPrice

from corptax.helpers import get_ratting_tax, generate_tax, lookup_invoice, notify_troika, get_ore_rarity_tax, unique, discordbot_send_embed_msg, discordbot_send_msg_remember
from corptax.models import MoonLedgerMember, CorpStats, DiscordNotification


from . import __title__
logger = get_extension_logger(__name__)

def generate_tax_ratting(start_date, end_date, notify):
    """Generate ratting tax for corporations in the accounted alliance."""
    esi = EsiClientProvider()
    fallback_ceo = getattr(settings, "FALLBACK_CEO", None)
    accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)

    year = end_date.strftime("%Y")
    month = end_date.strftime("%m")

    alliance_ratting_tax = getattr(settings, "RATTING_TAX", 0.1)
    renting_ratting_tax = getattr(settings, "RENT_RATTING_TAX", 0.15)

    try:
        alliance_corps = []
        for alliance in accounted_alliance:
            corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
            for x in corps:
                alliance_corps.append(x)
    except:
        logger.warning(f'Failed to make Alliance ESI query, existing')
        exit(1)
    
    for corp_id in alliance_corps:
        corporation_info = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
        corp_tax = round(corporation_info['tax_rate'], 2) * 100
        corp_tax_total = 0
        ratting_jornal = get_ratting_tax(start_date, end_date, corp_id)
        for entry in ratting_jornal:
            try:
                entry_tax = float(entry.amount) / corp_tax * 100
                corp_tax_total = corp_tax_total + entry_tax
            except Exception as e:
                logger.warning(f'{corp_id} division by zero in corp wallet entry: {entry}')
                continue
        if corporation_info['alliance_id'] == 741557221:
            corp_tax_total = round(corp_tax_total * alliance_ratting_tax)
            tax_rate = alliance_ratting_tax
        elif corporation_info['alliance_id'] == 99007906:
            corp_tax_total = round(corp_tax_total * renting_ratting_tax)
            tax_rate = renting_ratting_tax
        else:
            corp_tax_total = round(corp_tax_total * 0)
        if corp_tax_total > 0:
            try:
                corp_ceo = EveCharacter.objects.get(character_id=corporation_info['ceo_id'])
            except:
                logger.warning(f'Couldn\'t find CEO for corp {corp_id} setting invoice to Sophie Winter')
                corp_ceo = EveCharacter.objects.get(character_id=fallback_ceo)
                pass

            invoice_ref = "rattingtax" + str(corp_id) + str(year) + str(month)
            if corporation_info['alliance_id'] == 741557221:
                corp_name = corporation_info['name']
            elif corporation_info['alliance_id'] == 99007906:
                corp_name = "*" + corporation_info['name']
            else:
                corp_name = corporation_info['name']
            check_invoice = lookup_invoice(invoice_ref)
            if not check_invoice:
                logger.info(f'Generating invoice {invoice_ref} for {corp_id} amount {corp_tax_total} tax rate {tax_rate}')
                #bill = generate_invoice(corp_ceo, invoice_ref, corp_tax_total, tax_reason)
                generate_tax(corp_id, invoice_ref, corp_tax_total, corp_name, end_date)
            else:
                logger.warning(f'Invoice already exist {invoice_ref}')
        else:
            logger.warning(f'No bounty found for corp {corp_id}')
            if notify == True:
                notify_troika(f"Ratting Tax not generated for {corp_id}", f'No bounty found for corp {corp_id}', "info")
    return(True, "Ratting tax generation completed successfully.")


def generate_tax_moonmining(start_date, end_date, notify):
    """Generate moon mining tax for corporations in the accounted alliance."""
    fallback_ceo = getattr(settings, "FALLBACK_CEO", None)
    accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)

    year = end_date.strftime("%Y")
    month = end_date.strftime("%m")
    esi = EsiClientProvider()
    logger.info(f'Starting Moon Tax calculation for date {month}/{year}')

    try:
        alliance_corps = []
        for alliance in accounted_alliance:
            corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
            for x in corps:
                alliance_corps.append(x)
    except:
        logger.error(f'Failed to make Alliance ESI query, existing')
        exit(1)
    #logger.info(f'Corps in alliances {alliance_corps}')
    #alliance_corps = [98785282]
    for corp_id in alliance_corps:
        try:
            corp_req = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
            corp_ceo = EveCharacter.objects.get(character_id=corp_req['ceo_id'])
        except:
            logger.info(f'Couldn\'t find CEO for corp {corp_id} setting invoice to Sophie Winter')
            corp_ceo = EveCharacter.objects.get(character_id=fallback_ceo)
            pass

        invoice_ref = "moontax" + str(corp_id) + str(year) + str(month)
        corp_ledger_month = Ledger.objects.filter(corporation_id=corp_id, day__year=year, day__month=month)
        corp_tax_total = 0
        corp_member_list = []
        for x in corp_ledger_month:
            corp_tax_value = get_ore_rarity_tax(x.ore_type_id)
            corp_tax_amount = x.total_price * corp_tax_value
            corp_tax_total = corp_tax_total + corp_tax_amount
            corp_member_list.append(x.character_id)

        if corp_tax_total > 0:
            corp_member_list = unique(corp_member_list)

            member_list_tax = dict()
            for corp_member in corp_member_list:
                member_tax_total = 0
                member_ledger_month = Ledger.objects.filter(character_id=corp_member, day__year=year, day__month=month)
                for x in member_ledger_month:
                    member_tax_value = get_ore_rarity_tax(x.ore_type_id)
                    member_tax_amount = x.total_price * member_tax_value
                    member_tax_total = member_tax_total + member_tax_amount
                    day = x.day
                
                try:
                    character = esi.client.Character.get_characters_character_id(character_id=corp_member).results()
                except:
                    logger.error(f'Failed get ESI member {corp_member}')
                    continue

                try:
                    # Check if entry already exists for this month
                    entry = MoonLedgerMember.objects.get(date__month=month, date__year=year, corp_id=corp_id, character_id=corp_member)
                    entry.amount = member_tax_total
                    entry.date = day
                    entry.save()
                    logger.info(f'Updating existing mining ledger entry for {corp_member} in {month}/{year}')
                except:
                    logger.info(f'Creating new mining ledger entry for {corp_member} in {month}/{year}')
                    MoonLedgerMember.objects.create(date=day, corp_id=corp_id, character_id=corp_member, amount=member_tax_total, character_name=str(character['name']))

                
                update_member_tax = {str(character['name']):  member_tax_total}
                member_list_tax.update(update_member_tax)
            """ This I would only be need if I would use the Incoive module
            tax_reason = "Monthly Moon Mining Tax:\n"
            for key, value in member_list_tax.items():
                value = "{:,.2f}".format(value)
                tax_reason += str(key) + ": " + str(value) + "\n"
            """
            ### Generate Invoice
            logger.info(f'Generating invoice {invoice_ref} for {corp_id} amount {corp_tax_total} date {end_date}')
            #bill = generate_invoice(corp_ceo, invoice_ref, round(corp_tax_total), tax_reason)
            generate_tax(corp_id, invoice_ref, corp_tax_total, corp_req['name'], end_date)

        else:
            logger.info(f'No bill for {corp_id}')
    return(True, "Moon mining tax generation completed successfully.")

def generate_tax_moondrill(start_date, end_date, notify):
    """Generate moon drill tax for corporations in the accounted alliance."""
    year = end_date.strftime("%Y")
    month = end_date.strftime("%m")
    razor_tag = StructureTag.objects.get(name="Razor rent")
    edge_tag = StructureTag.objects.get(name="EDGE rent")
    drill_structures = Structure.objects.filter(eve_type_id=81826, tags=razor_tag) | Structure.objects.filter(eve_type_id=81826, tags=edge_tag)
    for drill in drill_structures:
        moon_product = MoonProduct.objects.filter(moon_id=drill.eve_moon_id)
        total_moon_value = 0
        if drill.eve_moon_id == None:
            logger.warning(f'Can not generate tax bill, moon is missing {drill}')
            if notify == True:
                notify_troika("Moon for drill missing", f'No moon found for {drill}', "danger")
            continue
        
        for x in moon_product:
            # 21888000 = 30.000(per hour) * 24 (hours) * 30.4 (days)
            ore_output_volume = 21888000 * x.amount
            ore_output = ore_output_volume / 10
            ore_material = EveTypeMaterial.objects.filter(eve_type=x.ore_type_id)
            for y in ore_material:
                ore_material_value = 0
                material_type = EveType.objects.get(id=y.material_eve_type_id)
                if material_type.eve_group_id not in [427]:
                    continue
                total_minerales = ore_output * y.quantity / 100
                price = EveMarketPrice.objects.get(eve_type=y.material_eve_type_id)
                ore_material_value = total_minerales * price.average_price
                total_moon_value = total_moon_value + ore_material_value
        razor_rent = Structure.objects.filter(id=drill.id, tags=razor_tag).count()
        edge_rent = Structure.objects.filter(id=drill.id, tags=edge_tag).count()
        if razor_rent > 0:
            tax_total = round(total_moon_value * 0.05)
            tax_class = "Razor"
        elif edge_rent > 0:
            tax_total = round(total_moon_value * 0.1)
            tax_class = "EDGE"
        else:
            continue
        logger.info(f'Generating invoice {str(drill.name)} Tax: {tax_total}, Tax class: {tax_class}')
        invoice_ref = "DRILL_" + str(drill.eve_moon_id) + str(year) + str(month)
        display_name = tax_class + " " + drill.name
        generate_tax('158202185', invoice_ref, tax_total, display_name, end_date)

def generate_tax_moonathanor(start_date, end_date, notify):
    """Generate tax for moon mining for athanor structures."""
    year = end_date.strftime("%Y")
    month = end_date.strftime("%m")
    razor_tag = StructureTag.objects.get(name="Razor rent")
    edge_tag = StructureTag.objects.get(name="EDGE rent")
    athanor_structures = Structure.objects.filter(eve_type_id=35835, tags=razor_tag) | Structure.objects.filter(eve_type_id=35835, tags=edge_tag)
    for athanor in athanor_structures:
        moon_product = MoonProduct.objects.filter(moon_id=athanor.eve_moon_id)
        total_moon_value = 0
        if athanor.eve_moon_id == None:
            logger.warning(f'Can not generate tax bill, moon is missing {athanor}')
            if notify == True:
                notify_troika("Moon for athanor missing", f'No moon found for {athanor}', "danger")
            continue
        for x in moon_product:
            # 21888000 = 30.000(per hour) * 24 (hours) * 30.4 (days)
            ore_output_volume = 21888000 * x.amount
            ore_output = ore_output_volume / 10
            ore_material = EveTypeMaterial.objects.filter(eve_type=x.ore_type_id)
            for y in ore_material:
                ore_material_value = 0
                material_type = EveType.objects.get(id=y.material_eve_type_id)
                # For Athanor we calculate all the output from the moon.
                #if material_type.eve_group_id not in [427]:
                #    continue
                total_minerales = ore_output * y.quantity / 100
                price = EveMarketPrice.objects.get(eve_type=y.material_eve_type_id)
                ore_material_value = total_minerales * price.average_price
                total_moon_value = total_moon_value + ore_material_value
        razor_rent = Structure.objects.filter(id=athanor.id, tags=razor_tag).count()
        edge_rent = Structure.objects.filter(id=athanor.id, tags=edge_tag).count()
        if razor_rent > 0:
            tax_total = round(total_moon_value * 0.1)
            tax_class = "Razor"
        elif edge_rent > 0:
            tax_total = round(total_moon_value * 0.2)
            tax_class = "EDGE"
        else:
            continue
        logger.info(f'Generating invoice {str(athanor.name)} Tax: {tax_total}, Tax class: {tax_class}')
        invoice_ref = "ATHANOR_" + str(athanor.eve_moon_id) + str(year) + str(month)
        display_name = tax_class + " " + athanor.name
        generate_tax('158202185', invoice_ref, tax_total, display_name, end_date)

def generate_corp_stats(start_date, end_date, notify):
    accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)
    logger.info(f'Checking corp stats for alliances {accounted_alliance} from {start_date} to {end_date}')
    esi = EsiClientProvider()
    all_active_corps = []
    for alliance in accounted_alliance:
        alliance_corps = []
        corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
        auth_alliance_info = EveAllianceInfo.objects.get(alliance_id=alliance)
        for x in corps:
            alliance_corps.append(x)
            all_active_corps.append(x)
        for corp_id in alliance_corps:
            logger.info(f'Checking corp {corp_id} for stats')
            auth_corp_info = EveCorporationInfo.objects.get(corporation_id=corp_id)
            auth_alliance_info = EveAllianceInfo.objects.get(id=auth_corp_info.alliance_id)
            esi_corp_info = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
            jornal = CorporationWalletJournalEntry.objects.filter(
                division__corporation__corporation__corporation_id=auth_corp_info.corporation_id, 
                date__gte=start_date, date__lte=end_date
            )
            corp_tax = round(esi_corp_info['tax_rate'], 2) * 100
            check_corp_journal = 0
            check_corp_ceo = 0
            check_audit_member = 0
            if "RZR Vote Corp" not in auth_corp_info.corporation_name:
                if len(jornal) < 1:
                    check_corp_journal = 1
                    logger.warning(f'No journal entries found for corp {corp_id} in date range {start_date} to {end_date}')
                try:
                    corp_ceo = EveCharacter.objects.get(character_id=auth_corp_info.ceo_id)
                except Exception as e:
                    logger.warning(f'Could not find CEO for corp {corp_id}')
                    check_corp_ceo = 1
                auth_members = EveCharacter.objects.filter(corporation_id=corp_id)
                for member in auth_members:
                    try:
                        audit_character = AuditCharacter.objects.get(eve_character_id=member.id)
                        check_audit_member = check_audit_member + 1
                    except:
                        pass
                try:
                    entry = CorpStats.objects.get(corp_id=corp_id)
                    entry.alliance_id = auth_alliance_info.alliance_id
                    entry.auth_member = len(auth_members)
                    entry.audit_member = check_audit_member
                    entry.corp_tax = corp_tax
                    entry.auth_ceo = check_corp_ceo
                    entry.corp_journal = check_corp_journal
                    entry.total_member = auth_corp_info.member_count
                    entry.save()
                except:
                    CorpStats.objects.create(corp_id=corp_id, alliance_id = auth_alliance_info.alliance_id,
                        corp_tax=corp_tax, auth_member=len(auth_members), 
                        audit_member=check_audit_member, auth_ceo=check_corp_ceo, corp_journal=check_corp_journal,
                        corp_name=auth_corp_info.corporation_name, total_member=auth_corp_info.member_count
                    )
    #clean up
    corp_remove = CorpStats.objects.exclude(corp_id__in=all_active_corps).values_list('corp_id', flat=True)
    if len(corp_remove) > 0:
        logger.info(f'to be removed {corp_remove}')
        CorpStats.objects.filter(corp_id__in=corp_remove).delete()


def check_corp_tax():
    alliances = getattr(settings, "ACCOUNTED_ALLIANCE", None)
    esi = EsiClientProvider()
    for alliance in alliances:
        alliance_corps = []
        corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
        for corp in corps:
            alliance_corps.append(corp)
        for corp in alliance_corps:
            esi_corp_info = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp).results()
            corp_tax = round(esi_corp_info['tax_rate'], 2) * 100
            if corp_tax < 1:
                logger.info(f'Corp {esi_corp_info["name"]} has a tax rate of {corp_tax}%')
                title = "Corp Tax setting alert"
                msg = f"{esi_corp_info['name']} has a tax rate of {corp_tax}%",
                notify_troika(
                    title=title,
                    msg=msg,
                    level="warning"
                )
                color = "red"
                channel = getattr(settings, "DISCORD_CORP_TAX_ALERT_CHANNEL", None)
                discordbot_send_msg_remember(
                    ownner=corp,
                    title=title,
                    msg=msg,
                    interval=2,
                    color=color,
                    channel=channel
                )


# Generated by Django 4.2.15 on 2024-09-27 08:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorpInvoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=20, verbose_name='Amount')),
                ('corp_name', models.CharField(max_length=128, verbose_name='CorpName')),
                ('tax_reason', models.CharField(max_length=128, verbose_name='CorpName')),
                ('corp_id', models.BigIntegerField(verbose_name='corp_id')),
                ('tax_date', models.DateField(verbose_name='tax_date')),
            ],
        ),
    ]



# Generated by Django 4.2.15 on 2024-11-21 11:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0007_alter_corpstats_corp_tax'),
    ]

    operations = [
        migrations.AddField(
            model_name='corpstats',
            name='corp_name',
            field=models.CharField(max_length=128, null=True, verbose_name='CorpName'),
        ),
        migrations.AddField(
            model_name='corpstats',
            name='total_member',
            field=models.IntegerField(blank=True, null=True, verbose_name='total_member'),
        ),
    ]



"""App Settings"""

# Django
from django.conf import settings
from django.apps import AppConfig

class CorpTaxConfig(AppConfig):
    """App configuration for CorpTax"""
    
    name = 'corptax'
    verbose_name = "Corp Tax"
    
    def ready(self):
        """Ready method to initialize app settings"""
        # Ensure the app is loaded and settings are configured
        if not hasattr(settings, 'CORPTAX'):
            settings.CORPTAX = {
                'ACCOUNTED_ALLIANCE': False,
            }






from django.core.management.base import BaseCommand
from allianceauth.services.hooks import get_extension_logger
from allianceauth.groupmanagement.models import AuthGroup
from allianceauth.authentication.models import CharacterOwnership, User
from corptax.helpers import notify_troika
from datetime import datetime, timedelta
logger = get_extension_logger(__name__)
class Command(BaseCommand):
    help = 'Spy Finder Assets'
    def handle(self, *args, **options):
        # cloud ring region id = 10000051
        #cd = AuthGroup.objects.filter(group="Corp Konzil der Drei")
        cd = AuthGroup.objects.all()
        for i in cd:
            print(vars(i))
        cd2 = CharacterOwnership.objects.all()
        for i in cd2:
            print(i)
            print(vars(i))

        user = User.objects.all()
        for x in user:
            print(vars(x))
        #print(dejar)
        notify_troika("Title", "Ein Test", "danger")


"""
App Models
"""

# Django
from django.db import models
from django.utils.translation import gettext as _
from django.db.models.functions import Now

class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """Meta definitions"""

        managed = False
        default_permissions = ()
        permissions = (("basic_access", "Can access this app"),)

class AdminView(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """Meta definitions"""

        managed = False
        default_permissions = ()
        permissions = (("admin_access", "Admin access"),)

class TroikaVie(models.Model):
    """Meta model for app permissions"""

    class Meta:
        """Meta definitions"""

        managed = False
        default_permissions = ()
        permissions = (("troika_access", "Troika access"),)
"""
class TaxPreview(models.Model):
    amount = models.DecimalField(_('Amount'), null=False, blank=False, decimal_places=2, max_digits=20)
    corp_name = models.CharField(_('CorpName'), null=False, blank=False, max_length=128)
    tax_reason = models.CharField(_('CorpName'), null=False, blank=False, max_length=128)
    corp_id = models.BigIntegerField(_('corp_id'), null=False, blank=False)
    tax_date = models.DateField(_('tax_date'), null=False, blank=False)


    class Meta:
        ordering = ['corp_name']
        verbose_name = _('Corp Tax')
        verbose_name_plural = _('Corp Taxes')
"""
"""
class CorpInvoice(models.Model):
    amount = models.DecimalField(_('Amount'), null=False, blank=False, decimal_places=2, max_digits=20)
    corp_name = models.CharField(_('CorpName'), null=False, blank=False, max_length=128)
    tax_reason = models.CharField(_('tax_reason'), null=False, blank=False, max_length=128)
    corp_id = models.BigIntegerField(_('corp_id'), null=False, blank=False)
    tax_date = models.DateField(_('tax_date'), null=False, blank=False)

    def __str__(self):
        return f"{self.amount} - {self.corp_id} - {self.tax_reason} - {self.tax_date}"
"""
# I wan to delete this
#class Invoice(models.Model):
#    corp_id = models.BigIntegerField(_('corp_id'), null=False, blank=False)
#    date = models.DateField(_('date'), null=False, blank=False)

class MoonLedgerMember(models.Model):
    corp_id = models.BigIntegerField(_('corp_id'), null=False, blank=False)
    character_id = models.BigIntegerField(_('character_id'), null=False, blank=False)
    character_name = models.CharField(_('CharacterName'), null=True, blank=True, max_length=128)
    date = models.DateField(_('date'), null=False, blank=False)
    amount = models.DecimalField(_('Amount'), null=False, blank=False, decimal_places=2, max_digits=20)

class CorpStats(models.Model):
    corp_id = models.BigIntegerField(_('corp_id'), null=False, blank=False)
    alliance_id = models.BigIntegerField(_('corp_id'), null=True, blank=False)
    corp_name = models.CharField(_('CorpName'), null=True, blank=False, max_length=128)
    total_member = models.IntegerField(_('total_member'), null=True, blank=True)
    auth_member = models.IntegerField(_('auth_member'), null=True, blank=True)
    audit_member = models.IntegerField(_('audit_member'), null=True, blank=True)
    auth_ceo = models.IntegerField(_('auth_ceo'), null=True, blank=True)
    corp_tax = models.DecimalField(_('corp_tax'), null=True, blank=True, decimal_places=2, max_digits=5)
    corp_journal = models.IntegerField(_('corp_journal'), null=True, blank=True)

class AllianceFinance(models.Model):
    date = models.DateField(_('date'), null=False, blank=False)
    income = models.DecimalField(_('income'), null=True, blank=True, decimal_places=2, max_digits=20)
    expense = models.DecimalField(_('expense'), null=True, blank=True, decimal_places=2, max_digits=20)
    reason = models.CharField(_('reason'), null=False, blank=False, max_length=128)
    description = models.CharField(_('reason'), null=True, blank=True, max_length=128)

class DiscordNotification(models.Model):
    is_sent = models.BooleanField(
        default=False,
        verbose_name=_("is sent"),
        help_text=_("True when this notification has been sent to Discord"),
    )
    owner = models.IntegerField(
        _('owner'),
        help_text=_("Owner of the message"),
    )
    time_sent = models.DateTimeField(
        default=True,
        help_text=("The time when the message has been sent")
    )
    discord_msg = models.CharField(
        _('discord_msg'),
        null=False,
        max_length=128,
        help_text=("The message what been sent to Discord")
    )

class CorpInvoice(models.Model):
    tax_reason = models.CharField(_('tax_reason'),
        null=False,
        blank=False,
        max_length=128
    )
    corp_id = models.BigIntegerField(_('corp_id'),
        null=False,
        blank=False
    )
    corp_name = models.CharField(_('CorpName'),
        null=False,
        blank=False,
        max_length=128
    )
    amount = models.DecimalField(_('Amount'),
        null=False,
        blank=False,
        decimal_places=2,
        max_digits=20
    )
    tax_date = models.DateField(_('tax_date'),
        null=False,
        blank=False
    )

    def __str__(self):
        return f"{self.amount} - {self.corp_id} - {self.tax_reason} - {self.tax_date}"
    
    def get_total_invoice(corp_id, tax_reason):
        total_tax = CorpInvoice.models.filter(corp_id=corp_id, tax_reason__in=tax_reason)
        return(total_tax)
        



# Generated by Django 4.2.15 on 2024-11-17 00:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0006_corpstats'),
    ]

    operations = [
        migrations.AlterField(
            model_name='corpstats',
            name='corp_tax',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True, verbose_name='corp_tax'),
        ),
    ]



import logging
import csv
from pprint import pprint
import operator as op
from inspect import getmembers, isfunction

from django.core.management.base import BaseCommand
from django.conf import settings

from eveuniverse.models import EveType, EveTypeMaterial, EveMarketPrice, EveMoon, EvePlanet, EveSolarSystem, EveConstellation, EveRegion
from moonmining.models import Moon, MoonProduct
from structures.models import Structure, StructureTag

logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'Generate Moon Drill Rental'
    def handle(self, *args, **options):
        print(f'Starting Moon Rental calculation')
        csv_filename = "/tmp/total_moon_value_without_minerals.csv"
        moons = Moon.objects.all()
        report = []
        for moon in moons:
            moon_location = EveMoon.objects.get(id=moon.eve_moon_id)
            eve_planet = EvePlanet.objects.get(id=moon_location.eve_planet_id)
            moon_system = EveSolarSystem.objects.get(id=eve_planet.eve_solar_system_id)
            moon_constellation = EveConstellation.objects.get(id=moon_system.eve_constellation_id)
            moon_region = EveRegion.objects.get(id=moon_constellation.eve_region_id)
            moon_product = MoonProduct.objects.filter(moon_id=moon.eve_moon_id)
            if moon_region.name != "Placid":
                print(moon_region.name)
                continue
            total_moon_value = 0
            material_output = ""
            for x in moon_product:
                # 21888000 = 30.000(per hour) * 24 (hours) * 30.4 (days)
                ore_output_volume = 21888000 * x.amount
                ore_output = ore_output_volume / 10
                ore_material = EveTypeMaterial.objects.filter(eve_type=x.ore_type_id)
                moon =  Moon.objects.get(eve_moon_id=moon.eve_moon_id)
                for y in ore_material:
                    ore_material_value = 0
                    material_type = EveType.objects.get(id=y.material_eve_type_id)
                    if material_type.eve_group_id not in [427]:
                        continue
                    total_minerales = ore_output * y.quantity / 100
                    input_mats_name = str(EveType.objects.get(id=y.material_eve_type_id))
                    price = EveMarketPrice.objects.get(eve_type=y.material_eve_type_id)
                    ore_material_value = total_minerales * price.average_price
                    total_moon_value = total_moon_value + ore_material_value
                    material_output = material_output + " / " + str(input_mats_name)
                    #print(input_mats_name, round(total_minerales))
            total_value = round(total_moon_value)
            razo_tax_total = round(total_moon_value * 0.05)
            edge_tax_total = round(total_moon_value * 0.1)
            drill_value = round(total_value * 0.4)
            print(f'{moon_region.name} / {str(moon.name)} / {str(moon.name)} Tax: {total_moon_value}')
            report_dict = {"Constalation": str(moon_system.eve_constellation),"System": str(moon_system.name), 
                           "Moon": str(moon.name), "Rarity Class": moon.rarity_class, 
                           "Total Value": total_value, "Drill value": drill_value, "Razor Tax": razo_tax_total,
                           "EDGE Tax": edge_tax_total, "Material": material_output
            }
            report.append(report_dict)

        fields = ['Constalation', 'System', 'Moon','Rarity Class' ,'Total Value', 'Drill value', 'Razor Tax', 'EDGE Tax', 'Material']
        with open(csv_filename, 'w') as file:
            writer = csv.DictWriter(file, fieldnames=fields, dialect='excel')
            writer.writeheader()
            writer.writerows(report)





import logging
import csv
import operator as op
from inspect import getmembers, isfunction
from datetime import datetime, timedelta, date
from dateutil.relativedelta import relativedelta

from django.core.management.base import BaseCommand
from django.utils import timezone
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag
from esi.clients import EsiClientProvider

from moonmining.models import MiningLedgerRecord as Ledger

from corptax.helpers import unique, lookup_invoice, generate_invoice, get_ore_rarity_tax, get_ratting_tax, generate_tax_preview

from corptax.models import TaxPreview

logger = logging.getLogger(__name__)

#### Moon Taxes
class Command(BaseCommand):
    help = 'Generate Moon Tax break down'
    def handle(self, *args, **options):
        accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)
        current_date = datetime.now()
        year = current_date.strftime("%Y")
        month = current_date.strftime("%m")
        esi = EsiClientProvider()
        logger.info(f'Starting Moon Tax calculation')

        try:
            alliance_corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=accounted_alliance).results()
        except:
            logger.warning(f'Failed to make Alliance ESI query, existing')
            exit(1)
        for corp_id in alliance_corps:
            invoice_ref = "moontax" + str(corp_id) + str(year) + str(month)
            corp_ledger_month = Ledger.objects.filter(corporation_id=corp_id, day__year=year, day__month=month)
            corp_tax_total = 0
            for x in corp_ledger_month:
                corp_tax_value = get_ore_rarity_tax(x.ore_type_id)
                corp_tax_amount = x.total_price * corp_tax_value
                corp_tax_total = corp_tax_total + corp_tax_amount

            if corp_tax_total > 0:
                try:
                    corp_name = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
                    logger.warning(f'Generating preview {invoice_ref} for {corp_id} amount {corp_tax_total}')
                    generate_tax_preview(corp_id, invoice_ref, corp_tax_total, corp_name['name'])
                except:
                    logger.warning(f'Failed generate preview: {corp_id}')
                
            else:
                logger.info(f'No moon mining for: {corp_id}')


from django.core.management.base import BaseCommand
from allianceauth.services.hooks import get_extension_logger
from esi.clients import EsiClientProvider
from app_utils.esi import fetch_esi_status
from corptax.helpers import discordbot_send_embed_msg
from corptax.models import DiscordNotification
from datetime import datetime, timedelta
logger = get_extension_logger(__name__)
class Command(BaseCommand):
    help = 'Bot test'
    def handle(self, *args, **options):
        if not fetch_esi_status().is_ok:
            logger.warning(f'ESI not working')
            quit()
        alliances = [99007906, 741557221]
        esi = EsiClientProvider()
        for alliance in alliances:
            alliance_corps = []
            corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
            for corp in corps:
                alliance_corps.append(corp)
            for corp in alliance_corps:
                esi_corp_info = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp).results()
                corp_tax = round(esi_corp_info['tax_rate'], 2) * 100
                if corp_tax < 1:
                    print(f'{corp} {corp_tax}')
                    title = "Corp Tax setting alert"
                    msg = f"{esi_corp_info['name']} Tax {corp_tax}%"
                    color = "red"
                    channel = 1347604527171371109
                    time_now = datetime.today()
                    before = time_now - timedelta(minutes=2)
                    check_sent = DiscordNotification.objects.filter(time_sent__gte=before, owner=corp, discord_msg=msg)
                    if not check_sent:
                        try:
                            discordbot_send_embed_msg(title, msg, color, channel)
                            DiscordNotification.objects.create(is_sent=True, owner=corp, discord_msg=msg, time_sent=time_now)
                            logger.info(f'Send discord message to channel {channel} message: {msg}')
                        except Exception as E:
                            logger.error(f'failed to send discord message {E}')
                            continue
                    else:
                        logger.info(f"we have already sent that msg: {msg}")
        
        
        
        
        
        
        
        
        
        
        
        #me = EveCharacter.objects.get(character_id=ceo_id)
        #print(vars(me))
        #print(me)
        #send_me = DiscordUser.objects.get(user_id=me.id)
        #print(vars(send_me))
        #print(send_me)
        #alldiscord = DiscordUser.objects.all()
        #for x in alldiscord:
        #    print(vars(x))



import csv
from inspect import getmembers, isfunction

from django.core.management.base import BaseCommand
from django.conf import settings

from allianceauth.services.hooks import get_extension_logger
from eveuniverse.models import EveType, EveTypeMaterial, EveMarketPrice, EveMoon, EvePlanet, EveSolarSystem, EveConstellation, EveRegion
from moonmining.models import Moon, MoonProduct
from structures.models import Structure, StructureTag

logger = get_extension_logger(__name__)
class Command(BaseCommand):
    help = 'Generate Moon Praisel'
    def handle(self, *args, **options):
        logger.info(f'Starting Moon Praisel')
        csv_filename = "/tmp/total_moon_value_without_minerals.csv"
        moons = Moon.objects.all()
        #moons = Moon.objects.filter(id=)
        report = []
        for moon in moons:
            moon_location = EveMoon.objects.get(id=moon.eve_moon_id)
            eve_planet = EvePlanet.objects.get(id=moon_location.eve_planet_id)
            moon_system = EveSolarSystem.objects.get(id=eve_planet.eve_solar_system_id)
            moon_constellation = EveConstellation.objects.get(id=moon_system.eve_constellation_id)
            moon_product = MoonProduct.objects.filter(moon_id=moon.eve_moon_id)
            moon_region = EveRegion.objects.get(id=moon_constellation.eve_region_id)
            if moon_region.name != "Cloud Ring":
                continue
            total_moon_value = 0
            material_output = ""
            print(f'{moon_region.name} / {moon_system.name} / {moon.name}')
            for x in moon_product:
                # 21888000 = 30.000(per hour) * 24 (hours) * 30.4 (days)
                ore_output_volume = 21888000 * x.amount
                ore_output = ore_output_volume / 10
                ore_material = EveTypeMaterial.objects.filter(eve_type=x.ore_type_id)
                moon =  Moon.objects.get(eve_moon_id=moon.eve_moon_id)
                for y in ore_material:
                    ore_material_value = 0
                    material_type = EveType.objects.get(id=y.material_eve_type_id)
                    if material_type.eve_group_id not in [427]:
                        continue
                    total_minerales = ore_output * y.quantity / 100
                    input_mats_name = str(EveType.objects.get(id=y.material_eve_type_id))
                    price = EveMarketPrice.objects.get(eve_type=y.material_eve_type_id)
                    ore_material_value = total_minerales * price.average_price
                    total_moon_value = total_moon_value + ore_material_value
                    material_output = material_output + " / " + str(input_mats_name)
                    print(input_mats_name, round(total_minerales))
            total_value = round(total_moon_value)
            razo_tax_total = round(total_moon_value * 0.05)
            edge_tax_total = round(total_moon_value * 0.1)
            drill_value = round(total_value * 0.4)
            #print(f'{str(moon.name)} / {str(moon.name)} Tax: {total_moon_value}')
            report_dict = {"Constalation": str(moon_system.eve_constellation),"System": str(moon_system.name), 
                           "Moon": str(moon.name), "Rarity Class": moon.rarity_class, 
                           "Total Value": total_value, "Drill value": drill_value, "Razor Tax": razo_tax_total,
                           "EDGE Tax": edge_tax_total, "Material": material_output
            }
            report.append(report_dict)

        fields = ['Constalation', 'System', 'Moon','Rarity Class' ,'Total Value', 'Drill value', 'Razor Tax', 'EDGE Tax', 'Material']
        #with open(csv_filename, 'w') as file:
            #writer = csv.DictWriter(file, fieldnames=fields, dialect='excel')
            #writer.writeheader()
            #writer.writerows(report)





import logging
import csv
import operator as op
from inspect import getmembers, isfunction
from datetime import datetime, timedelta, date
from dateutil.relativedelta import relativedelta

from django.core.management.base import BaseCommand
from django.utils import timezone
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from app_utils.logging import LoggerAddTag
from esi.clients import EsiClientProvider

from moonmining.models import MiningLedgerRecord as Ledger

from corptax.helpers import unique, lookup_invoice, generate_invoice, get_ore_rarity_tax, get_ratting_tax

from corptax.models import TaxPreview

logger = logging.getLogger(__name__)

#### Moon Taxes
class Command(BaseCommand):
    help = 'Generate Moon Tax break down'
    def handle(self, *args, **options):
        fallback_ceo = getattr(settings, "FALLBACK_CEO", None)
        accounted_alliance = getattr(settings, "ACCOUNTED_ALLIANCE", None)
        last_month = datetime.now() - relativedelta(months=1)
        last_year = last_month.strftime("%Y")
        last_month = last_month.strftime("%m")
        #last_month="09"
        current_date = date.today()
        year = current_date.year
        month = current_date.month
        esi = EsiClientProvider()
        logger.info(f'Starting Moon Tax calculation')

        try:
            alliance_corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=accounted_alliance).results()
        except:
            logger.warning(f'Failed to make Alliance ESI query, existing')
            exit(1)
        for corp_id in alliance_corps:
            try:
                corp_req = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
                corp_ceo = EveCharacter.objects.get(character_id=corp_req['ceo_id'])
            except:
                logger.warning(f'Couldn\'t find CEO for corp {corp_id} setting invoice to Sophie Winter')
                corp_ceo = EveCharacter.objects.get(character_id=fallback_ceo)
                pass

            invoice_ref = "moontax" + str(corp_id) + str(last_year) + str(last_month)
            corp_ledger_month = Ledger.objects.filter(corporation_id=corp_id, day__year=last_year, day__month=last_month)
            corp_tax_total = 0
            corp_member_list = []
            for x in corp_ledger_month:
                corp_tax_value = get_ore_rarity_tax(x.ore_type_id)
                corp_tax_amount = x.total_price * corp_tax_value
                corp_tax_total = corp_tax_total + corp_tax_amount
                corp_member_list.append(x.character_id)

            if corp_tax_total > 0:
                corp_member_list = unique(corp_member_list)

                member_list_tax = dict()
                for corp_member in corp_member_list:
                    member_tax_total = 0
                    member_ledger_month = Ledger.objects.filter(character_id=corp_member, day__year=last_year, day__month=last_month)
                    for x in member_ledger_month:
                        member_tax_value = get_ore_rarity_tax(x.ore_type_id)
                        member_tax_amount = x.total_price * member_tax_value
                        member_tax_total = member_tax_total + member_tax_amount

                    character = esi.client.Character.get_characters_character_id(character_id=corp_member).results()
                    update_member_tax = {str(character['name']):  member_tax_total}
                    member_list_tax.update(update_member_tax)

                tax_reason = "Monthly Moon Mining Tax:\n"
                for key, value in member_list_tax.items():
                    value = "{:,.2f}".format(value)
                    tax_reason += str(key) + ": " + str(value) + "\n"

                ### Generate Invoice
                try:
                    corp_name = esi.client.Corporation.get_corporations_corporation_id(corporation_id=corp_id).results()
                    #corp_name = EveCharacter.objects.get(character_id=corp_req['name'])
                except:
                    continue
                logger.info(f'Generating invoice for {corp_id} amount {corp_tax_total}')
                logger.info(f'Generating invoice {invoice_ref} for {corp_id} amount {corp_tax_total}')
                try:
                    entry = TaxPreview.objects.get(corp_id=corp_id, tax_reason=invoice_ref)
                    #logger.warning(f'{entry.tax_reason}')
                    entry.field = corp_tax_total
                    entry.save()
                except Exception as E:
                    logger.warning(f'Ohhhh BIN DAAA?? {E}')
                    entry = TaxPreview.objects.create(amount=corp_tax_total, corp_name=corp_name['name'], tax_reason=invoice_ref, corp_id=corp_id, tax_date=timezone.now())
                if lookup_invoice(invoice_ref) is True:
                    logger.warning(f'Invoice already exist {invoice_ref}')
                    continue
            else:
                logger.info(f'No bill for {corp_id}')


"""App Configuration"""

# Django
from django.apps import AppConfig

# AA corptax 
from corptax import __version__


class ExampleConfig(AppConfig):
    """App Config"""

    name = "corptax"
    label = "corptax"
    verbose_name = f"corptax v{__version__}"




from datetime import datetime
from esi.clients import EsiClientProvider
from allianceauth.services.hooks import get_extension_logger
from allianceauth.notifications.models import Notification
from allianceauth.authentication.models import User
from corptax.models import CorpInvoice, CorpStats, AllianceFinance, DiscordNotification
from corptools.models import CorporationWalletJournalEntry
from invoices.models import Invoice
from eveuniverse.models import EveType, EveMarketPrice, EveSolarSystem
from memberaudit.models import CharacterMiningLedgerEntry, Character, CharacterSkillpoints, CharacterSkill
from allianceauth.eveonline.models import EveCharacter
from django.conf import settings
from django.utils import timezone
from django.db.models import Q
from django.apps import apps
from datetime import timedelta
import operator as op
import csv
import tempfile

from . import __title__
logger = get_extension_logger(__name__)

def unique(list1):
    unique_list = []
    for x in list1:
        if op.countOf(unique_list, x) == 0:
            unique_list.append(x)
    return(unique_list)

def lookup_invoice(invoice_ref):
    invoice = Invoice.objects.filter(invoice_ref = invoice_ref).exists()
    return(invoice)

###Usign the Incoive module
def generate_invoice(character, ref, amount, message):
    due_date_days = getattr(settings, "DUE_DATE_DAYS", 14)
    due = timezone.now() + timedelta(days=due_date_days)
    bill = Invoice(character=character,
        amount=amount,
        invoice_ref=ref,
        note=message,
        due_date=due)
    bill.save()

def get_ore_rarity_tax(type_id):
    ore_type = EveType.objects.get(id=type_id)
    if ore_type.eve_group_id == 1923:
        tax = getattr(settings, "EXCEPTIONAL_MOON_TAX", 0.35)
    elif ore_type.eve_group_id == 1922:
        tax = getattr(settings, "RARE_MOON_TAX", 0.15)
    elif ore_type.eve_group_id == 1921:
        tax = getattr(settings, "UNCOMMON_MOON_TAX", 0.025)
    elif ore_type.eve_group_id == 1920:
        tax = getattr(settings, "COMMON_MOON_TAX", 0.025)
    elif ore_type.eve_group_id == 1884:
        tax = getattr(settings, "UBIQUITOUS_MOON_TAX", 0.025)
    else:
        tax = 0
    return(tax)

def get_ratting_tax(start_date, end_date, corp_id):
    ratting_jornal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["bounty_prizes", "ess_escrow_transfer"], date__gte=start_date, date__lte=end_date)
    return(ratting_jornal)

def generate_tax(corp_id, invoice_ref, corp_tax_total, corp_name, invoice_date):
    try:
        entry = CorpInvoice.objects.get(corp_id=corp_id, tax_reason=invoice_ref)
        entry.amount = corp_tax_total
        entry.tax_reason = invoice_ref
        entry.tax_date = invoice_date
        entry.save()
    except:
        entry = CorpInvoice.objects.create(amount=corp_tax_total, corp_name=corp_name, tax_reason=invoice_ref, corp_id=corp_id, tax_date=invoice_date)
    return(entry)


def download_corp_member_ledger_csv(corp_id):
    csv_filename = tempfile.NamedTemporaryFile(delete=False)
    logger.info(f'Generating csv export for member ledger {csv_filename}')
    characters = Character.objects.all()
    report = []
    for character in characters:
        ledger = CharacterMiningLedgerEntry.objects.filter(character_id=character.id)
        auth_character = EveCharacter.objects.get(id=character.eve_character_id)
        if auth_character.corporation_id != corp_id:
            continue
        for char_ledger in ledger:
            ore_material = EveType.objects.get(id=char_ledger.eve_type_id)
            system = EveSolarSystem.objects.get(id=char_ledger.eve_solar_system_id)
            try:
                price = EveMarketPrice.objects.get(eve_type=char_ledger.eve_type_id)
                price = price.average_price
                total_price = price * char_ledger.quantity
            except:
                price = 0
                total_price = 0
            report_dict = {"date": char_ledger.date, "character": auth_character, "system": system,"type": ore_material, "quantity": char_ledger.quantity, "price": round(price,2), "total_price": round(total_price, 2)}
            report.append(report_dict)
    fields = ['date', 'character', 'system', 'type', 'quantity', 'price', 'total_price']
    with open(csv_filename.name, 'w') as file:
        writer = csv.DictWriter(file, fieldnames=fields, dialect='excel')
        writer.writeheader()
        writer.writerows(report)
    logger.info(f'CSV generated {corp_id}')
    return(csv_filename)

## Guess this is not used anymore, but I leave it here for now
def get_update_corp_stats(corp_id, auth_members, check_audit_member, corp_tax, check_corp_ceo, check_corp_journal, auth_corp_info):
    try:
        entry = CorpStats.objects.get(corp_id=corp_id)
        entry.auth_member = len(auth_members)
        entry.audit_member = check_audit_member
        entry.corp_tax = corp_tax
        entry.auth_ceo = check_corp_ceo
        entry.corp_journal = check_corp_journal
        entry.total_member = auth_corp_info.member_count
        entry.save()
    except:
        CorpStats.objects.create(corp_id=corp_id, corp_tax=corp_tax, auth_member=len(auth_members), 
            audit_member=check_audit_member, auth_ceo=check_corp_ceo, corp_journal=check_corp_journal,
            corp_name=auth_corp_info.corporation_name, total_member=auth_corp_info.member_count
        )

    return()

# Copilots idea
#
# def finance_calculation(start_date, end_date):
#     corp_id = 158202185
#     logger.info(f'Start finance calculation: {start_date} {end_date}')

#     # Helper to build and append finance dicts
#     def add_finance_entry(finance_list, reason, description, journal, income_filter=None, expense_filter=None):
#         entry = {
#             "reason": reason,
#             "description": description,
#             "income": 0,
#             "expense": 0
#         }
#         for j in journal:
#             if expense_filter and expense_filter(j):
#                 entry["expense"] += j.amount
#             elif income_filter and income_filter(j):
#                 entry["income"] += j.amount
#             elif not income_filter and not expense_filter:
#                 # Default: all positive is income, negative is expense
#                 if j.amount < 0:
#                     entry["expense"] += j.amount
#                 else:
#                     entry["income"] += j.amount
#         finance_list.append(entry)

#     # Query sets
#     planatary_journal = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["planetary_import_tax", "planetary_export_tax"], amount__gt=0,
#         date__gte=start_date, date__lte=end_date
#     )
#     industry_journal = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["industry_job_tax", "reaction", "copying", "manufacturing"],
#         date__gte=start_date, date__lte=end_date
#     )
#     reprocessing_journal = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["reprocessing_tax"],
#         date__gte=start_date, date__lte=end_date
#     )
#     brokers_journal = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["brokers_fee"],
#         date__gte=start_date, date__lte=end_date
#     )
#     contract_price_journal = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["contract_price", "contract_price_payment_corp"],
#         date__gte=start_date, date__lte=end_date
#     )
#     market_transaction = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["market_transaction"],
#         date__gte=start_date, date__lte=end_date
#     )
#     structure_gate_jump = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["structure_gate_jump"], amount__gt=0,
#         date__gte=start_date, date__lte=end_date
#     )
#     corporation_account_withdrawal_srp = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["corporation_account_withdrawal"], amount__lt=0,
#         date__gte=start_date, date__lte=end_date,
#         reason__contains="SRP",
#     ).exclude(second_party_id=corp_id)
#     corporation_account_income = CorporationWalletJournalEntry.objects.filter(
#         division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["corporation_account_withdrawal", "player_donation"], second_party_id=corp_id, amount__gt=0,
#         date__gte=start_date, date__lte=end_date
#     ).exclude(first_party_id=corp_id)
#     corporation_account_withdrawal = CorporationWalletJournalEntry.objects.filter(
#         Q(division__corporation__corporation__corporation_id=corp_id,
#         ref_type__in=["corporation_account_withdrawal"]) & Q(first_party_id=corp_id, amount__lt=0) &
#         Q(date__gte=start_date, date__lte=end_date) & ~Q(reason__contains="SRP") & ~Q(second_party_id=corp_id)
#     )

#     finance_list = []

#     # Add entries
#     add_finance_entry(finance_list, "planatary", "Planatary", planatary_journal, income_filter=lambda e: True)
#     add_finance_entry(finance_list, "industry", "Industry", industry_journal)
#     add_finance_entry(finance_list, "reprocessing", "Reprocessing", reprocessing_journal)
#     add_finance_entry(finance_list, "structure_gate_jump", "Ansiblex Jump Gate", structure_gate_jump)
#     add_finance_entry(finance_list, "brokers_fee", "Brokers Fee", brokers_journal)
#     add_finance_entry(
#         finance_list, "contract_price", "Contracts others", contract_price_journal,
#         income_filter=lambda e: e.amount > 0 and e.division_id not in [9, 11, 12, 13],
#         expense_filter=lambda e: e.amount < 0 and e.division_id not in [9, 11, 12, 13]
#     )
#     add_finance_entry(
#         finance_list, "contract_price_buyback", "Contracts Buyback", contract_price_journal,
#         income_filter=lambda e: e.amount > 0 and e.division_id == 9,
#         expense_filter=lambda e: e.amount < 0 and e.division_id == 9
#     )
#     add_finance_entry(
#         finance_list, "contract_price_metenox_skyhook", "Contracts Sell Skyhook/Metenox", contract_price_journal,
#         income_filter=lambda e: e.amount > 0 and e.division_id == 13,
#         expense_filter=lambda e: e.amount < 0 and e.division_id == 13
#     )
#     add_finance_entry(
#         finance_list, "contract_price_home_def", "Contracts Home Def ", contract_price_journal,
#         income_filter=lambda e: e.amount > 0 and e.division_id == 12,
#         expense_filter=lambda e: e.amount < 0 and e.division_id == 12
#     )
#     add_finance_entry(
#         finance_list, "contract_price_fuel", "Contracts Fuel", contract_price_journal,
#         income_filter=lambda e: e.amount > 0 and e.division_id == 11,
#         expense_filter=lambda e: e.amount < 0 and e.division_id == 11
#     )
#     add_finance_entry(finance_list, "market_transaction", "Market Transaction", market_transaction)
#     add_finance_entry(
#         finance_list, "srp", "SRP Phonix Fleet", corporation_account_withdrawal_srp,
#         expense_filter=lambda e: e.division_id == 10
#     )
#     add_finance_entry(
#         finance_list, "srp_razor", "SRP Razor Fleet", corporation_account_withdrawal_srp,
#         expense_filter=lambda e: e.division_id == 12
#     )
#     add_finance_entry(
#         finance_list, "account_withdrawal", "Account Withdrawal", corporation_account_withdrawal,
#         expense_filter=lambda e: True
#     )
#     add_finance_entry(
#         finance_list, "account_income", "ISK Transfer to RHC", corporation_account_income,
#         income_filter=lambda e: e.division_id != 9
#     )
#     add_finance_entry(
#         finance_list, "account_income_buyback", "Income from Buyback", corporation_account_income,
#         income_filter=lambda e: e.division_id == 9
#     )

#     # Save to DB
#     for entry in finance_list:
#         try:
#             db_entry = AllianceFinance.objects.get(reason=entry['reason'], date=start_date)
#             db_entry.income = entry['income']
#             db_entry.expense = entry['expense']
#             db_entry.description = entry['description']
#             db_entry.save()
#         except Exception:
#             AllianceFinance.objects.create(
#                 reason=entry['reason'],
#                 description=entry['description'],
#                 income=entry['income'],
#                 expense=entry['expense'],
#                 date=start_date
#             )

def finance_calculation(start_date, end_date):
    corp_id = 158202185
    logger.info(f'Start finance calculation: {start_date} {end_date}')
    planatary_journal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["planetary_import_tax", "planetary_export_tax"], amount__gt=0,
        date__gte=start_date, date__lte=end_date
    )
    industry_journal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["industry_job_tax", "reaction", "copying", "manufacturing"],
        date__gte=start_date, date__lte=end_date
    )
    reprocessing_journal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["reprocessing_tax"],
        date__gte=start_date, date__lte=end_date
    )
    brokers_journal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["brokers_fee"],
        date__gte=start_date, date__lte=end_date
    )
    contract_price_journal = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["contract_price", "contract_price_payment_corp"],
        date__gte=start_date, date__lte=end_date
    )
    market_transaction = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["market_transaction"],
        date__gte=start_date, date__lte=end_date
    )
    structure_gate_jump = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["structure_gate_jump"], amount__gt=0,
        date__gte=start_date, date__lte=end_date
    )
    corporation_account_withdrawal_srp = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["corporation_account_withdrawal"], amount__lt=0,
        date__gte=start_date, date__lte=end_date,
        reason__contains="SRP",
    ).exclude(second_party_id=158202185)
    corporation_account_income = CorporationWalletJournalEntry.objects.filter(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["corporation_account_withdrawal", "player_donation"], second_party_id=158202185, amount__gt=0,
        date__gte=start_date, date__lte=end_date
    ).exclude(first_party_id=158202185)
    corporation_account_withdrawal = CorporationWalletJournalEntry.objects.filter(Q(division__corporation__corporation__corporation_id=corp_id,
        ref_type__in=["corporation_account_withdrawal"]) & Q(first_party_id=158202185, amount__lt=0) &
        Q(date__gte=start_date, date__lte=end_date) & ~Q(reason__contains="SRP") & ~Q(second_party_id=158202185))
    
    finance_list = []
    
    finance_dict = {}
    finance_dict["reason"] = "planatary"
    finance_dict["description"] = "Planatary"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in planatary_journal:
        finance_dict["income"] = finance_dict["income"] + entry.amount
    print(finance_dict["income"])
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "industry"
    finance_dict["description"] = "Industry"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in industry_journal:
        if entry.amount < 0:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        else:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "reprocessing"
    finance_dict["description"] = "Reprocessing"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in reprocessing_journal:
        if entry.amount < 0:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        else:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "structure_gate_jump"
    finance_dict["description"] = "Ansiblex Jump Gate"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in structure_gate_jump:
        if entry.amount < 0:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        else:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "brokers_fee"
    finance_dict["description"] = "Brokers Fee"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in brokers_journal:
        if entry.amount < 0:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        else:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "contract_price"
    finance_dict["description"] = "Contracts others"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in contract_price_journal:
        if entry.amount < 0 and not entry.division_id == 9 and not entry.division_id == 11 and not entry.division_id == 12 and not entry.division_id == 13:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        elif entry.amount > 0 and not entry.division_id == 9 and not entry.division_id == 11 and not entry.division_id == 12 and not entry.division_id == 13:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "contract_price_buyback"
    finance_dict["description"] = "Contracts Buyback"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in contract_price_journal:
        if entry.amount < 0 and entry.division_id == 9:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        elif entry.amount > 0 and entry.division_id == 9:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "contract_price_metenox_skyhook"
    finance_dict["description"] = "Contracts Sell Skyhook/Metenox"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in contract_price_journal:
        if entry.amount < 0 and entry.division_id == 13:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        elif entry.amount > 0 and entry.division_id == 13:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "contract_price_home_def"
    finance_dict["description"] = "Contracts Home Def "
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in contract_price_journal:
        if entry.amount < 0 and entry.division_id == 12:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        elif entry.amount > 0 and entry.division_id == 12:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "contract_price_fuel"
    finance_dict["description"] = "Contracts Fuel"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in contract_price_journal:
        if entry.amount < 0 and entry.division_id == 11:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        elif entry.amount > 0 and entry.division_id == 11:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "market_transaction"
    finance_dict["description"] = "Market Transaction"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in market_transaction:
        if entry.amount < 0:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
        else:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "srp"
    finance_dict["description"] = "SRP Phonix Fleet"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in corporation_account_withdrawal_srp:
        if entry.division_id == 10:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "srp_razor"
    finance_dict["description"] = "SRP Razor Fleet"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in corporation_account_withdrawal_srp:
        if entry.division_id == 12:
            finance_dict["expense"] = finance_dict["expense"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "account_withdrawal"
    finance_dict["description"] = "Account Withdrawal"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in corporation_account_withdrawal:
        finance_dict["expense"] = finance_dict["expense"] + entry.amount
    finance_list.append(finance_dict)

    finance_dict = {}
    finance_dict["reason"] = "account_income"
    finance_dict["description"] = "ISK Transfer to RHC"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in corporation_account_income:
        if  entry.division_id != 9:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)
    
    finance_dict = {}
    finance_dict["reason"] = "account_income_buyback"
    finance_dict["description"] = "Income from Buyback"
    finance_dict["income"] = 0
    finance_dict["expense"] = 0
    for entry in corporation_account_income:
        if entry.division_id == 9:
            finance_dict["income"] = finance_dict["income"] + entry.amount
    finance_list.append(finance_dict)
    
    for entry in finance_list:
        #logger.debug(f'{entry}')
        try:
            db_entry = AllianceFinance.objects.get(reason=entry['reason'], date=start_date)
            db_entry.income = entry['income']
            db_entry.expense = entry['expense']
            db_entry.description = entry['description']
            db_entry.save()
        except:
            db_entry = AllianceFinance.objects.create(reason=entry['reason'], description= entry['description'], income=entry['income'], expense=entry['expense'], date=start_date)

def discordbot_send_embed_msg(title, msg, color, channel):
    def discord_bot_active():
        return apps.is_installed('aadiscordbot')
    if discord_bot_active():
        from discord import Embed, Color
        from aadiscordbot.tasks import send_message
        if color == "red":
            embedded_msg = Embed(title=title,
                    description=msg,
                    color=Color.red())
        elif color == "yellow":
            embedded_msg = Embed(title=title,
                    description=msg,
                    color=Color.yellow())
        else:
            embedded_msg = Embed(title=title,
                    description=msg,
                    color=Color.green())
        if not channel == None:
            discord_respone = send_message(channel_id=channel, embed=embedded_msg)
        else:
            discord_respone = False
        return(discord_respone)
    else:
        return(False)

def discordbot_send_msg_remember(owner, title, msg, channel, interval, color):
    time_now = datetime.today()
    before = time_now - timedelta(days=interval)
    check_sent = DiscordNotification.objects.filter(time_sent__gte=before, owner=owner, discord_msg=msg)
    if not check_sent:
        try:
            discordbot_send_embed_msg(title, msg, color, channel)
            DiscordNotification.objects.create(is_sent=True, owner=owner, discord_msg=msg, time_sent=time_now)
            logger.info(f'Send discord message to channel {channel} message: {msg}')
        except Exception as E:
            logger.error(f'failed to send discord message {E}')
    else:
        logger.info(f"we have already sent that msg: {msg}")


def notify_troika(title, msg, level):
    user_list = getattr(settings, "TROIKA_NOTIFY", [])
    for troika in user_list:
        user = User.objects.get(username=troika)
        if level == "warning":
            Notification.objects.notify_user(
                    user=user,
                    title=title,
                    message=msg,
                    level=Notification.Level.WARNING
                )
        elif level == "danger":
            Notification.objects.notify_user(
                    user=user,
                    title=title,
                    message=msg,
                    level=Notification.Level.DANGER
                )
        else:
            Notification.objects.notify_user(
                    user=user,
                    title=title,
                    message=msg,
                    level=Notification.Level.INFO
                )

def young_cyno_chars():
    esi = EsiClientProvider()
    time_now = datetime.now()
    member_chars = Character.objects.all()
    for member_char in member_chars:
        auth_char = EveCharacter.objects.get(id=member_char.eve_character_id)
        try:
            #21603 is Cyno skill
            cyno_skill = CharacterSkill.objects.get(eve_type_id=21603, character_id=member_char.id)
            eve_char = esi.client.Character.get_characters_character_id(character_id=auth_char.character_id).results()
        except:
            continue
        eve_char_birthday = datetime.strptime(str(eve_char['birthday']), '%Y-%m-%d %H:%M:%S+00:00')
        character_age = time_now - eve_char_birthday
        #7776000 is 90 days old in sec.
        try:
            if round(character_age.total_seconds()) < 7776000 and cyno_skill.trained_skill_level > 0 and auth_char.alliance_id == 741557221:
                logger.info(f'Name: {member_char.name}, Age: {character_age}, has Cyno skill at level 5')
                result = discordbot_send_msg_remember(
                    owner=auth_char.character_id,
                    title="Character with Cyno skill V",
                    msg=f'Name: {member_char.name}, Age: {character_age}, has Cyno skill at level 5',
                    interval=2,
                    color="red",
                    channel=getattr(settings, "DISCORD_SPY_ALERT", None)
                )
        except Exception as e:
            logger.error(f'Failed to send cyno alert via discor: {e}')
            pass


from django.core.management.base import BaseCommand
from allianceauth.services.hooks import get_extension_logger
from corptax.models import CorpInvoice
from memberaudit.models import CharacterWalletJournalEntry as char_journal
from memberaudit.models import Character, CharacterSkillpoints, CharacterSkill, Location, CharacterAsset
from eveuniverse.models import EveSolarSystem, EveConstellation
from esi.clients import EsiClientProvider
from allianceauth.eveonline.models import EveCharacter
from app_utils.esi import fetch_esi_status
from datetime import datetime, timedelta
logger = get_extension_logger(__name__)
class Command(BaseCommand):
    help = 'Spy Finder'
    def handle(self, *args, **options):
        esi = EsiClientProvider()
        if not fetch_esi_status().is_ok:
            logger.warning(f'ESI not working')
            quit()
        alliances = [99007906, 741557221]
        alliance_corps = []
        for alliance in alliances:
            esi_alliance_corps = esi.client.Alliance.get_alliances_alliance_id_corporations(alliance_id=alliance).results()
            for x in esi_alliance_corps:
                alliance_corps.append(x)
        print(f'Player transfer/donation larger then 10b in last 30 day\'s')
        time_now = datetime.now()
        date_range= time_now - timedelta(days=30)
        journal = char_journal.objects.filter(amount__gte=10000000000, ref_type__contains="player_", date__gte=date_range)
        for entry in journal:
            try:
                first_party = esi.client.Character.get_characters_character_id(character_id=entry.first_party_id).results()
            except Exception as e:
                print(f'{entry.first_party_id} {e}')
                continue
            first_party_corp = esi.client.Corporation.get_corporations_corporation_id(corporation_id=first_party['corporation_id']).results()
            #print(first_party_corp['alliance_id'])
            if first_party_corp['alliance_id'] is not None:
                first_party_alliance = esi.client.Alliance.get_alliances_alliance_id(alliance_id=first_party['alliance_id']).results()
            
            try:
                second_party = esi.client.Character.get_characters_character_id(character_id=entry.second_party_id).results()
            except Exception as e:
                print(f'{entry.second_party_id} {e}')
                continue
            second_party_corp = esi.client.Corporation.get_corporations_corporation_id(corporation_id=second_party['corporation_id']).results()
            if second_party_corp['alliance_id'] is not None:
            #if second_party_corp['alliance_id'] or len(second_party_corp['alliance_id']) > 0:
                second_party_alliance = esi.client.Alliance.get_alliances_alliance_id(alliance_id=second_party['alliance_id']).results()
            #print(first_party_corp)
            if first_party['corporation_id'] != second_party['corporation_id'] and (first_party['corporation_id'] in alliance_corps or second_party['corporation_id'] in alliance_corps):
                print(f"{entry.date} {entry.amount:,} FROM: {first_party['name']}/{first_party_corp['name']} TO: {second_party['name']}/{second_party_corp['name']}")
            #print(first_party)
            
        ##Skill check
        #21603 cyno skill
        print("")
        print(f'Character with cyno level 5 skill and less then 90 days old')
        member_chars = Character.objects.all()
        for member_char in member_chars:
            #total skull point
            total_skill = CharacterSkillpoints.objects.get(character_id=member_char.id)
            auth_char = EveCharacter.objects.get(id=member_char.eve_character_id)
            try:
                #21603 is Cyno skill
                cyno_skill = CharacterSkill.objects.get(eve_type_id=21603, character_id=member_char.id)
                eve_char = esi.client.Character.get_characters_character_id(character_id=auth_char.character_id).results()
            except:
                continue
            eve_char_birthday = datetime.strptime(str(eve_char['birthday']), '%Y-%m-%d %H:%M:%S+00:00')
            character_age = time_now - eve_char_birthday
            #7776000 is 90 days old in sec.
            try:
                if round(character_age.total_seconds()) < 7776000 and cyno_skill.trained_skill_level > 0 and auth_char.alliance_id == 741557221:
                    print(f'Name: {member_char.name}, Age: {character_age}, has Cyno skill at level 5')
            except Exception as e:
                print(e)
                pass
        
        #### Assets search
        asset_region_id = 10000051
        for member_char in member_chars:
            auth_char = EveCharacter.objects.get(id=member_char.eve_character_id)
            eve_char = esi.client.Character.get_characters_character_id(character_id=auth_char.character_id).results()
            character_age = time_now - eve_char_birthday

            assets = CharacterAsset.objects.filter(character_id=member_char.id, location_id__gt=1)
            count_items = 0
            if round(character_age.total_seconds()) > 15552000:
                continue
            for asset in assets:
                location = Location.objects.get(id=asset.location_id)
                #print(vars(asset))
                #print(vars(location))
                try:
                    system = EveSolarSystem.objects.get(id=location.eve_solar_system_id)
                    constellation = EveConstellation.objects.get(id=system.eve_constellation_id)
                except:
                    continue
                if constellation.eve_region_id == asset_region_id:
                    count_items = count_items + 1
                    #print(f'{auth_char} / {asset} / {location}')
            if count_items < 5 and not count_items == 0:
                print(f'{member_char.name} / total items = {count_items}')

            cd = CorpInvoice.get_total_invoice(96505959, "moontax")
            for i in cd:
                print(i)


# Generated by Django 4.2.15 on 2024-11-16 14:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptax', '0005_moonledgermember_character_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorpStats',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('corp_id', models.BigIntegerField(verbose_name='corp_id')),
                ('auth_member', models.IntegerField(blank=True, null=True, verbose_name='auth_member')),
                ('audit_member', models.IntegerField(blank=True, null=True, verbose_name='audit_member')),
                ('auth_ceo', models.IntegerField(blank=True, null=True, verbose_name='auth_ceo')),
                ('corp_tax', models.DecimalField(blank=True, decimal_places=2, max_digits=3, null=True, verbose_name='corp_tax')),
                ('corp_journal', models.IntegerField(blank=True, null=True, verbose_name='corp_journal')),
            ],
        ),
    ]



import logging
import csv
from pprint import pprint
import operator as op
from inspect import getmembers, isfunction

from django.core.management.base import BaseCommand
from django.conf import settings

from allianceauth.eveonline.models import EveCharacter
from eveuniverse.models import EveType, EveTypeMaterial, EveMarketPrice
from moonmining.models import Moon, MoonProduct
from structures.models import Structure, StructureTag
from memberaudit.models import CharacterMiningLedgerEntry, Character

logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'Generate Moon Drill Rental'
    def handle(self, *args, **options):
        csv_filename = "/tmp/personal_ledger.csv"
        characters = Character.objects.all()
        report = []
        for character in characters:
            ledger = CharacterMiningLedgerEntry.objects.filter(character_id=character.id)
            auth_character = EveCharacter.objects.get(id=character.eve_character_id)
            if auth_character.corporation_id != 98785282:
                continue
            for char_ledger in ledger:
                ore_material = EveType.objects.get(id=char_ledger.eve_type_id)
                try:
                    price = EveMarketPrice.objects.get(eve_type=char_ledger.eve_type_id)
                    price = price.average_price
                    total_price = price * char_ledger.quantity
                except:
                    price = 0
                    total_price = 0
                #print(char_ledger.date, char_ledger.quantity, ore_material, price, total_price)
                report_dict = {"date": char_ledger.date, "character": auth_character, "type": ore_material, "quantity": char_ledger.quantity, "price": round(price,2), "total_price": round(total_price, 2)}
                report.append(report_dict)

        fields = ['date', 'character', 'type', 'quantity', 'price', 'total_price']
        with open(csv_filename, 'w') as file:
            writer = csv.DictWriter(file, fieldnames=fields, dialect='excel')
            writer.writeheader()
            writer.writerows(report)







import logging
from datetime import date
from django.db.models import Sum
from django.core.management.base import BaseCommand
from corptax.models import CorpInvoice
logger = logging.getLogger(__name__)
class Command(BaseCommand):
    help = 'View test'
    def handle(self, *args, **options):
        current_date = date.today()
        year = current_date.year
        month = current_date.month
        tax_record_moon = CorpInvoice.objects.filter(tax_reason__contains='moon', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
        tax_record_ratting = CorpInvoice.objects.filter(tax_reason__contains='ratting', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
        tax_record_drill = CorpInvoice.objects.filter(tax_reason__contains='DRILL', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
        tax_record_athanor = CorpInvoice.objects.filter(tax_reason__contains='ATHANOR_', tax_date__month=month, tax_date__year=year).order_by('-tax_date', 'corp_name')
        tax_month_list = CorpInvoice.objects.filter(tax_reason__contains='moon').values('tax_date').distinct().order_by('-tax_date')[1:12]
        #total_per_corp = CorpInvoice.objects.filter(tax_date__month=month, tax_date__year=year).values('-tax_date', 'corp_name')
        tax_date = str(month) + "/" + str(year)
        total_moon = 0
        for entry in tax_record_moon:
            total_moon = total_moon + entry.amount
        total_ratting = 0
        for entry in tax_record_ratting:
            total_ratting = total_ratting + entry.amount
        total_drill = 0
        for entry in tax_record_drill:
            total_drill = total_drill + entry.amount
        total_athanor = 0
        for entry in tax_record_athanor:
            total_athanor = total_athanor + entry.amount
        total_per_corp = CorpInvoice.objects.filter(tax_date__month=month, tax_date__year=year).values('corp_name').annotate(total_amount=Sum('amount')).order_by('-corp_name')
        for entry in total_per_corp:
            logger.debug(f"Corp: {entry['corp_name']} Total: {entry['total_amount']}")
            print(f"Corp: {entry['corp_name']} Total: {entry['total_amount']}")

